function J = imclsmooth( I, param )


param = setparameter( param );

global DEBUG;
DEBUG = param.DEBUG;


% BEGIN DEBUG ------------------------
if DEBUG
	V_log = cell(3,1); % color-line vector of init, aligned, smoothed.
	R_log = cell(2,1); % residual components of init and smoothed
	CL_log = cell(2,1); % color-line components of init and smoothed
end
% END DEBUG --------------------------

[sy, sx, ncolor] = size( I );

% Create a color-line vector field
V = zeros( sy, sx, ncolor ); % color line vector field
M = zeros( sy, sx, ncolor ); % mean (of distribution) field
D = zeros( sy, sx ); % standard deviation (of distribution) field


% initialization of the output image
J = I;

for ite = 1:param.nite
	
	%
	%
	%
	% Calculate Color-Line Vector Filed (CLVF)
	%
	%
	%
	% calculate color orientation on each pixel
	% with their neighboring MxN pixels
	%
% 	tic
	[V, D, M] = colorvf( J, param.hws );
% 	toc
	
	% BEGIN DEBUG ------------------------
	if DEBUG
		V_log{1} = V;
	end
	% END DEBUG --------------------------
	
	
	%
	% Alignment of color vector orientation
	%
	nLv = nextpow2( max( size(V,1), size(V,2) ) ); % maximum resolution
	% nLv = 5;
	nMG = 5;
	
% 	tic
	V = alignvdir( V, M, D, nLv, nMG );
% 	toc
	
	% BEGIN DEBUG ------------------------
	if DEBUG
		V_log{2} = V;
	end
	% END DEBUG --------------------------
	
	
	%
	% Orthonormal vector field regularization
	%
	
% 	tic
	V = orthvf( V, ...
		param.orth_hws, param.orth_std, param.orth_enhance, ...
		param.orth_preserve, param.orth_dt, param.orth_nite );
% 	toc
	
	% BEGIN DEBUG ------------------------
	if DEBUG
		V_log{3} = V;

% 		figure(2), imshow( 0.5*(1+[V_log{1}, V_log{2}, V_log{3}]) );
% 		title('Color-Line Vector Field.  From left,  Initial CLVF,  Sign-aligned CLVF and Smoothed CLVF');
	end
	% END DEBUG --------------------------
	
	
	%
	%
	%
	% Color decomposition
	%
	%
	%
	[I_CL, V_resi, I_resi] = decomp_color( J, M, V );
	
	% BEGIN DEBUG ------------------------
	if DEBUG
		CL_log{1} = I_CL;
		R_log{1} = I_resi;
		
% 		figure(3), imshow( [M, repmat(0.5*(CL_log{1}+1),[1 1 3]), repmat(0.5*(R_log{1}+1),[1 1 3])] );
% 		title( 'Decomposed components.  weighted mean,  color-line,  residual' );
		
	end
	% END DEBUG --------------------------
	
	
	%
	%
	%
	% Smoothing for the Color-Line component
	%
	%
	%
	
% 	%
% 	% Non-Local mean
% 	%
% 	para_sm.w_radius = 4; % radius of the filter window
% 	para_sm.k_radius = 5; % radius of the similarity search
% 	para_sm.std_range = 0.03;
% 	I_CL_sm = smooth_nonlocal( I_CL, para_sm );
	
% 	%
% 	% WLS filter
% 	%
% 	I_CL = 0.5*(I_CL+1);
% 	I_CL = min( max( I_CL, 0 ), 1 );
% 	I_CL_sm = smooth_wls_cg( I_CL, 0.5, [], 10 );
% 	I_CL_sm = 2*I_CL_sm - 1;
	
	%
	% Anisotropic Diffusion
	%
	para_sm.hws = 2;   % half window size of gaussian function
	para_sm.std = 1.5; % standard deviation
	para_sm.enhance = 0.1; % enchancement degree. (enhance > 0)
	para_sm.dt = 0.25; % time step
	para_sm.preserve = 0.2;
	para_sm.nite = 10; % number of iterations
	
	I_CL_sm = smooth_diffusion( I_CL, ...
		para_sm.hws, para_sm.std, para_sm.enhance, ...
		para_sm.preserve, para_sm.dt, para_sm.nite );
	
% 	figure(100), imshow( 0.5*([I_CL, I_CL_sm, I_CL-I_CL_sm] +1 ) );
% 	title( 'Results of smoothing for the CL component.  original,  smoothed,  error' );
	
	
	%
	%
	%
	% Smoothing for the residual component
	%
	%
	%
	
	I_resi_sm = smooth_resi_comp( I_resi );
	
% 	%
% 	% Non-Local mean
% 	%
% 	para_sm_resi.w_radius = 3; % radius of the filter window
% 	para_sm_resi.k_radius = 5; % radius of the similarity search
% 	para_sm_resi.std_range = 0.04;
% 	I_resi_sm = smooth_nonlocal( I_resi, para_sm_resi );
	
% 	%
% 	% WLS filter
% 	%
% 	I_resi_sm = smooth_wls_cg( I_resi, 0.5 );
	
% 	%
% 	% Anisotropic Diffusion
% 	%
% 	I_resi_sm= smooth_diffusion( I_resi, ...
% 			para_sm.hws, para_sm.std, para_sm.enhance, ...
% 			para_sm.preserve, para_sm.dt, para_sm.nite );
	
% 	figure(101), imshow( 0.5*([I_resi, I_resi_sm, I_resi-I_resi_sm]+1) );
% 	title( 'Results of smoothing for the residual component.  original,  smoothed,  error' );
	
	

	%
	%
	%
	% Color recomposition
	%
	%
	%
	J = comp_color( M, V, I_CL_sm, V_resi, I_resi_sm );
	
end

end

%
%
% -------------------------------------------------------------------------
%
% Parameter setting
%
% -------------------------------------------------------------------------
%
%

function param = setparameter( param_ )
%
% PARAM = SETPARAMETER( PARAM_MANUAL ) overwrites default parameters PARAM
% by user specified ones PARAM_MANUAL
%
param.DEBUG = 0;
param.nite = 3; % number of main iterations
param.hws = 4;  % half window size for the calculation of color orientations

%
% PDE parameters of orthnormal vector field regularization
%
% smoothing parameters for gradient vector calculation
param.orth_hws = 2; % window size of gaussian function
param.orth_std = 1.5; % standard deviation

% diffusion parameters
param.orth_enhance = 0.2; % enchancement degree. (enhance > 0)
param.orth_dt = 0.25; % time step
param.orth_preserve = 0.2;
param.orth_nite = 10; % number of iterations

names = fieldnames( param_ );

for i = 1:length( names )
	name = names{i};
	param = setfield( param, name, getfield( param_, name ) );
end

end


%
%
% -------------------------------------------------------------------------
%
% Smoothing functions
%
% -------------------------------------------------------------------------
%
%

function J = orthvf( I, hws, wstd, enhance, preserve, dt, nIter )
%
% edge preserving smoothing for the multi channel image
% using Coherence Enhancing Flow
%
% J = impdeimpdecef( I, hws, wstd, permeation, nIter )
%
% inputarguments
%
%	I : Multi channel image
%
%
% hws, wstd : smoothing parameters for gradient vector calculation
%
%   hws      : half window size of gaussian filter
%   wstd     : standard deviation
%
%   enhance  : enchancement degree. (enhance > 0)
%   preserve : preserve original
%   dt       : time step
%   nIter    : number of iterations
%
%
% output arguments
%
%	J : smoothed image
%
%
%
% REFERENCE
%
% D. Tschumperle, R. Deriche.
% Vector-valued image regularization with PDE's :
% A Common framework for different applications
% CVPR'2003
%

scale = enhance * 255^2;


[sy, sx, ncolor] = size( I );
J = I;


% derivation operator mask
Kx = 0.5 * [-1, 0, 1];
Ky = Kx';

Kxx = [1 -2 1];
Kyy = Kxx';
Kxy = 0.25 * [1 0 -1; 0 0 0; -1 0 1];

% smoothing filter
Kg = fspecial( 'gaussian', 2*hws+1, wstd );

DH = zeros( size( J ) );

for iItr = 1:nIter
	
	% gradient of an image
	[Ix Iy] = gradient_( J, Kx, Ky );
	
	% tensor of gradients
	[G11, G12, G21, G22] = structure_tensor( Ix, Iy, Kg );
	
	% Calculate the direction and the magnitude of gradients
	[R_min, R_max, Ux, Uy, Vx, Vy] = gradv( G11, G12, G21, G22 );
	
	R_min = scale * R_min;
	R_max = scale * R_max;
	
	% set parameters for the diffusion tensor
	R2 = 1 ./ ( 1 + R_min + R_max );
	R1 = sqrt( R2 );
	
	[D11, D12, D21, D22] = diffusion_tensor( R1, R2, Ux, Uy, Vx, Vy );
	
	% calculate trace( DH )
	[ Ixx, Ixy, Iyx, Iyy ] = hessian_( J, Kxx, Kxy, Kyy );
	
	for i = 1:ncolor
		DH(:,:,i) = ( D11.*Ixx(:,:,i) + D12.*Ixy(:,:,i) ) + ( D21.*Iyx(:,:,i) + D22.*Iyy(:,:,i) );
	end
	
	% orthonormal constraint
	L = preserve * ( J - I ) - (1-preserve) * DH;
	
	% update
	J = J + dt * ( repmat( dot( L, J, 3 ), [1 1 3] ) .* J - L );
	
	% normalization
	J = J .* repmat( 1./sqrt( sum( J.*J, 3 ) ), [1 1 3] );
end

end

%
% -------------------------------------------------------------------------
%

function [ Ix Iy ] = gradient_( I, Kx, Ky )
Ix = imfilter( I, Kx, 'replicate' );
Iy = imfilter( I, Ky, 'replicate' );
end

%
% -------------------------------------------------------------------------
%

function [ Ixx, Ixy, Iyx, Iyy ] = hessian_( I, Kxx, Kxy, Kyy )
Ixx = imfilter( I, Kxx, 'replicate' );
Ixy = imfilter( I, Kxy, 'replicate' );
Iyx = Ixy;
Iyy = imfilter( I, Kyy, 'replicate' );
end

%
% -------------------------------------------------------------------------
%

function 	[ G11, G12, G21, G22 ] = structure_tensor( Ix, Iy, Kg )
G11 = sum( Ix.*Ix, 3 );
G12 = sum( Ix.*Iy, 3 );
G22 = sum( Iy.*Iy, 3 );

G11 = imfilter( G11, Kg, 'replicate' );
G12 = imfilter( G12, Kg, 'replicate' );
G21 = G12;
G22 = imfilter( G22, Kg, 'replicate' );
end

%
% -------------------------------------------------------------------------
%

function [ D11, D12, D21, D22 ] = diffusion_tensor( R1, R2, Ux, Uy, Vx, Vy )
D11 = R1 .* Ux.*Ux + R2 .* Vx.*Vx;
D12 = R1 .* Ux.*Uy + R2 .* Vx.*Vy;
D22 = R1 .* Uy.*Uy + R2 .* Vy.*Vy;
D21 = D12;
end

%
% -------------------------------------------------------------------------
%

function [ R_min, R_max, Ux, Uy, Vx, Vy ]...
	= gradv( G11, G12, G21, G22 )
%
% Calculate the direction and the magnitude of gradients
% by using eigen-value analysis
%

A = G11 + G22;
B = G11 - G22;
C = G12 .* G21;
D = sqrt( B.^2 + 4 * C );

R_min = 0.5 * ( A - D );
R_max = 0.5 * ( A + D );

R_min = max( R_min, 0 );
R_max = max( R_max, 0 );

t_max = atan2( R_max - G11, G12 );
Vx = cos( t_max );
Vy = sin( t_max );

Ux = -Vy;
Uy =  Vx;

end

%
%
% -------------------------------------------------------------------------
%
% Image composition / decomposition functions
%
% -------------------------------------------------------------------------
%
%
function [I_cl, V_resi, I_resi] = decomp_color( I, M, V_cl )


[sy, sx, n] = size( I );
V_resi = zeros( sy, sx, n ); % residual


V_diff = I - M;
I_cl = sum( V_diff.*V_cl, 3 ); % inner product at each pixel

for i = 1:n
	V_resi(:,:,i) = V_diff(:,:,i) - I_cl .* V_cl(:,:,i);
end

%
% normalization
%
% norm
I_resi = sqrt( sum( V_resi.*V_resi, 3 ) );

% avoid 0 division
V_resi = V_resi + eps;
I_resi = I_resi + eps;
for i = 1:n
	V_resi(:,:,i) = V_resi(:,:,i) ./ I_resi;
end

end

%
% -------------------------------------------------------------------------
%

function R = comp_color( M, V_cl, I_cl, V_resi, I_resi )

[sy, sx, n] = size( M );
R = zeros( sy, sx, n );

for i = 1:n
	R(:,:,i) = M(:,:,i) + I_cl.*V_cl(:,:,i) + I_resi.*V_resi(:,:,i);
end

end

%
%
% -------------------------------------------------------------------------
%
% Calculation of the initial color line vector field
%
% -------------------------------------------------------------------------
%
%

function [ V, D, M ] = colorvf( I, hws )
%
% [ V, D, M ] = colorvf( I_ex, szImg, hws )
% [ V, D, M ] = colorvf( I, szImg, hws, V, D )
%
% This function calculate the color direction (vector) on each pixels
% with their neighboring MxN pixels.
% As a result, color vector field V and magnitude field D are obtained.
%
%
% Arguments
%
% I : Input image
%
% hws : Half window size of the calculation window
%
%
% V : color vector field
% D : magnitude vector field
%

[V, D, M] = impca( I, hws );

% var to std
D = sqrt( D );

end

%
% -------------------------------------------------------------------------
%

function [v, d, m] = wpca( X, W_s )
%
% Color-line calcuration by weighted PCA
%
% [v, d, m] = wpca( X )
%
% Input arguments
%
%   X : local color image.
%
% Output arguments
%
%   v : color-line vector
%   d : variance in color-line direction
%   m : weighted mean
%


% 2nd weight that associates with
% distance from the initial center
sigma = 0.6^2;


[sy, sx, ncolor] = size( X );
X = reshape( X, [sy*sx, ncolor] );


% initial center
xg = median( X, 1 ); % median works well for noisy images
% xg = mean( X, 1 );

% weight by distance from the centroid
DD = centerize( X, xg );
DD = sum( DD.*DD, 2 );

W_r = exp( (-0.5/sigma)*DD );
W = W_r .* W_s;

W = W + 1e-6; % regularization (avoid 0 dividing)
W = W / sum(W);


m = W'*X;
m = m / sum( W );

% weighted cov
HX = centerize( X, m );

WHX = HX;
for i = 1:ncolor
	WHX(:,i) = HX(:,i) .* W;
end

C_w = (1/(1-sum(W.*W))) * (HX'*WHX);


% eigen analysis
[V, D] = eig( C_w );

% color-line vector and variance as
% the first principal component
[d, ind] = max( diag(D) );
v = V(:, ind );

end

%
% -------------------------------------------------------------------------
%

function [v, d, m] = pca( X )
%
% Color-line calcuration by PCA
%
% [v, d, m] = pca( X )
%
% Input arguments
%
%   X : local color image.
%
% Output arguments
%
%   v : color-line vector
%   d : variance in color-line direction
%   m : weighted mean
%

[sy, sx, ncolor] = size( X );
X = reshape( X, [sy*sx, ncolor] );

% initial center
m = mean( X, 1 );
C = cov( X );

% eigen analysis
[V, D] = eig( C );

% color-line vector and variance as
% the first principal component
[d, ind] = max( diag(D) );
v = V(:, ind );

end

%
% -------------------------------------------------------------------------
%

function D = centerize( X, y )

D = X;
for i = 1:size( X, 2 )
	D(:,i) = D(:,i) - y(i);
end

end

%
%
% -------------------------------------------------------------------------
%
% Align vector direction of the vector field
%
% -------------------------------------------------------------------------
%
%

function V = alignvdir( V, M, D, nLv, nMG )
%
% S = alignvdir_preflip( V )
%
% aligns the direction of each vector by changing the sign
% so that the neighboring vector directions become smooth.
%


% nLv = nextpow2( min( sy, sx ) );
nite = 3;


S_list = cell( nLv, 1 );
V_list = cell( nLv, 1 );
D_list = cell( nLv, 1 );

%
% initial flip by mean color vectors
%
V = pre_flip( V, M );

%
% Multi grid
%
V0 = V;

for ite = 1:nMG
	
	%
	% down sampling
	%
	for i = 1:nLv-1
		
		S = sign_flip( V, D, nite );
		VS = V .* repmat( S, [1 1 3] );
		
		V_list{i} = V;
		S_list{i} = S;
		D_list{i} = D;
		
		V = downsample_VF( VS, D );
		D = downsample( D );
	end
	
	% coarse resolution
	S = sign_flip( V, D, nite );
	
	
	%
	% up sampling
	%
	for i = nLv-1:-1:1
		
		S_u = upsample( S, D );
		
		S_c = S_list{ i };
		V   = V_list{ i };
		D   = D_list{ i };
		
		% size adjustment
		[ty, tx] = size(S_c);
		S = S_c .* S_u(1:ty, 1:tx);
		
		VS = V .* repmat( S, [1 1 3] );
		
		S = S .* sign_flip( VS, D, nite );
	end
	
	V0 = V0 .* repmat( S, [1 1 3] );
	V = V0;
	
end

end

%
% -------------------------------------------------------------------------
%

function S = sign_flip( V, D, nite )

[sy, sx, nneigbolor] = size( V );
S = ones( sy, sx );

for i = 1:nite
	if i == 1
		VS = V;
	else
		VS = V .* repmat( S, [1 1 nneigbolor] );
	end
	S = S .* sign_flip_by_neigb( VS, D );
end

end

%
% -------------------------------------------------------------------------
%

function S = sign_flip_by_neigb( V, D )
%
% Correct the direction of center vector
% to fit the 8 neighboring vectors
%

K = ones( 3 );
Vj = imfilter( V, K, 0 );
S = sign( sum( V.*Vj, 3 ) - 1 );

% K = ones( 3 );
% Vj = imfilter( V.*repmat(D,[1 1 3]), K, 0 );
% S = sign( sum( V.*Vj, 3 ) - D );


%
% Weighted version
%
% alpha = 50/iLv;
% beta = -1/alpha^2;
%
% wfunc = @( X ) exp( beta * X.*X );
% D = D + eps;
%
% % Calcuration of inner products
% % among 4 adjacent pixels
% vv_v = sum( V(1:end-1,:,:) .* V(2:end,:,:), 3 );
% vv_h = sum( V(:,1:end-1,:) .* V(:,2:end,:), 3 );
%
% W_v = D(1:end-1,:) - D(2:end,:);
% W_h = D(:,1:end-1) - D(:,2:end);
%
% W_v = wfunc( W_v );
% W_h = wfunc( W_h );
%
% W1 = padarray( W_v, [1, 0], 0, 'pre' );
% W2 = padarray( W_v, [1, 0], 0, 'post' );
% W3 = padarray( W_h, [0, 1], 0, 'pre' );
% W4 = padarray( W_h, [0, 1], 0, 'post' );
%
% VV = ...
% 	+ W1 .* padarray( vv_v, [1, 0], 0, 'pre'  ) ... % top
% 	+ W2 .* padarray( vv_v, [1, 0], 0, 'post' ) ... % bottom
% 	+ W3 .* padarray( vv_h, [0, 1], 0, 'pre'  ) ... % left
% 	+ W4 .* padarray( vv_h, [0, 1], 0, 'post' );    % right
% clear vv_v vv_h;
%
%
% % diagonal 4 adjacent pixels
% vv_d1 = sum( V(1:end-1,1:end-1,:) .* V(2:end,2:end,:), 3 );
% vv_d2 = sum( V(2:end,1:end-1,:) .* V(1:end-1,2:end,:), 3 );
%
% W_d1 = D(1:end-1,1:end-1) - D(2:end,2:end);
% W_d2 = D(2:end,1:end-1,:) - D(1:end-1,2:end,:);
%
% W_d1 = wfunc( W_d1 );
% W_d2 = wfunc( W_d2 );
%
% W1 = padarray( W_d1, [1 1], 0, 'pre' );
% W2 = padarray( W_d1, [1 1], 0, 'post' );
% W3 = padarray( padarray( W_d2, [1 0], 0, 'pre' ), [0 1], 'post' );
% W4 = padarray( padarray( W_d2, [1 0], 0, 'post'), [0 1], 'pre'  );
%
% VV = VV + ...
% 	+ W1 .* padarray( vv_d1, [1 1], 0, 'pre'  ) ... % top-left
% 	+ W2 .* padarray( vv_d1, [1 1], 0, 'post' ) ... % bottom-right
% 	+ W3 .* padarray( padarray( vv_d2, [1 0], 0, 'pre'  ), [0 1], 0, 'post' ) ... % top-right
% 	+ W4 .* padarray( padarray( vv_d2, [1 0], 0, 'post' ), [0 1], 0, 'pre'  );    % bottom-left
% clear vv_d1 vv_d2;
%
% S = sign( VV );


end

%
% -------------------------------------------------------------------------
%

function V_d = downsample_VF( V, D )
%
% Down sample the image.
% The size of image is reduced to half.
%

%
% resizing
%
[sy, sx, ndim] = size( V );

sy_re = ceil(sy/2);
sx_re = ceil(sx/2);

V = V .* repmat( D+eps, [1 1 ndim] );


V_ex = padarray( V, [2*sy_re-sy, 2*sx_re-sx], 'replicate', 'post' );

%
% filtering
%
a = 0.4;
q = 0.25;
b = q - a*0.5;
K = [b q a q b]';
K = K*K';
K = K / sum(K(:));

% K = [1 2 1; 2 4 2; 1 2 1];
% K = K / sum(K(:));

V_ex = imfilter( V_ex, K, 'replicate' );
V_d = V_ex(1:2:end,1:2:end,:);


if ndim == 3
	%
	% normalization
	%
	V_norm = sqrt( sum( V_d.*V_d, 3 ) );
	
	for i = 1:ndim
		V_d(:,:,i) = (V_d(:,:,i) + eps) ./ (V_norm + eps);
	end
end

end

%
% -------------------------------------------------------------------------
%

function I_d = downsample( I )
%
% Down sample the image.
% The size of image is reduced to half.
%

%
% resizing
%
[sy, sx, ndim] = size( I );

sy_re = ceil(sy/2);
sx_re = ceil(sx/2);

I_ex = padarray( I, [2*sy_re-sy, 2*sx_re-sx], 'replicate', 'post' );

%
% filtering
%
a = 0.4;
q = 0.25;
b = q - a*0.5;
K = [b q a q b]';
K = K*K';
K = K / sum(K(:));

% K = [1 2 1; 2 4 2; 1 2 1];
% K = K / sum(K(:));

I_ex = imfilter( I_ex, K, 'replicate' );
I_d = I_ex(1:2:end,1:2:end,:);

end

%
% -------------------------------------------------------------------------
%

function I_u = upsample( I, D )
%
% The size of image is expanded to double.
%

%
% resizing
%
[sy, sx, ndim] = size( I );

sy_ex = 2*sy;
sx_ex = 2*sx;

I = I .* D;

I_u = zeros( sy_ex, sx_ex, ndim );
I_u(1:2:end, 1:2:end, :) = I;

%
% filtering
%
a = 0.4;
q = 0.25;
b = q - a*0.5;
K = [b q a q b]';
K = K*K';
K = K / sum(K(:));

% K = [1 2 1; 2 4 2; 1 2 1];
% K = K / sum(K(:));

I_u = imfilter( I_u, K, 'replicate' );

I_u( I_u >= 0 ) = 1;
I_u( I_u <  0 ) = -1;

end

%
% -------------------------------------------------------------------------
%

function V = pre_flip( V, M )
%
% The direction (sign) of each "color line" vector is flipped
% so as to point in the same direction as the mean color of a pixel.
%

M_norm = sqrt( sum( M.*M, 3 ) );

M_unit = M;
for i = 1:3
	M_unit(:,:,i) = M_unit(:,:,i) ./ M_norm;
end

S = sign( sum( V.*M_unit, 3 ) );

for i = 1:3
	V(:,:,i) = V(:,:,i) .* S;
end

end

%
% -------------------------------------------------------------------------
%

function [V, D, M] = impca( I, r )
%
% for a RGB color image
%
% input arguments
%
% -     I : input image
% -     r : radius of filter
% - pfunc_std : function pointer
%
%
% output arguments
%
% -     J : output image
%

[sy, sx, ~] = size( I );
npix = sy*sx;

epsilon = 0.00001; % regularization for covariance matrix


% an integral image for the number of pixels within window
N_inv = 1.0 ./ imboxfilter_c( ones(sy,sx), r, r );

% get a pointer and give short name for readability
I1 = I(:,:,1);
I2 = I(:,:,2);
I3 = I(:,:,3);

% an integral image for the mean calculation
M1 = N_inv .* imboxfilter_c( I1, r, r );
M2 = N_inv .* imboxfilter_c( I2, r, r );
M3 = N_inv .* imboxfilter_c( I3, r, r );

% integral images for the covariance calculation
% covariance at each pixel
%
%     |C11, C12, C13|   |C11, C12, C13|
% C = |C21, C22, C23| = |     C22, C23|
%     |C31, C32, C33|   |diag      C33|
%
C11 = N_inv .* imboxfilter_c( I1.*I1, r, r ) - M1.*M1;
C12 = N_inv .* imboxfilter_c( I1.*I2, r, r ) - M1.*M2;
C13 = N_inv .* imboxfilter_c( I1.*I3, r, r ) - M1.*M3;
C22 = N_inv .* imboxfilter_c( I2.*I2, r, r ) - M2.*M2;
C23 = N_inv .* imboxfilter_c( I2.*I3, r, r ) - M2.*M3;
C33 = N_inv .* imboxfilter_c( I3.*I3, r, r ) - M3.*M3;

%
% regularization for C
%
C11 = C11 + epsilon;
C22 = C22 + epsilon;
C33 = C33 + epsilon;


%
% svd at each pixel
%
nite = 30;
tole = 0.00001;

[V1, V2, V3, D] = ...
	imeig_power_c( C11, C12, C13, C12, C22, C23, C13, C23, C33, [], [], [], nite, tole );

V = cat( 3, V1, V2, V3 );
M = cat( 3, M1, M2, M3 );

end

%
% -------------------------------------------------------------------------
%

function J = smooth_resi_comp( I )

%
% McClure weighting
%
XX = I.*I;
W = XX./(0.002+XX);

IW = I .* W;

%
% smoothing
%
% gaussian
% J = imfilter( IW, fspecial( 'gaussian', 5, 0.5 ), 'replicate' );

% median
J = medfilt2( IW, [3, 3] );

% do nothing
% J = IW;

end

