%**************************************************************************
%Written by Antony Lam (antony@nii.ac.jp)
%National Institute of Informatics, Tokyo, Japan
%September 28, 2013

%Copyright (c) 2013 National Institute of Informatics

%This software is released under the terms of the GNU General Public
%License version 3 as published by the Free Software Foundation.
%A copy of the license is included with this software in the file gpl.txt
%and it may also be obtained at http://www.gnu.org/licenses/.

%The software is provided as is without any warranty or guaranteed support
%whatsoever. The National Institute of Informatics cannot be held
%responsible for its use, misuse, or functionality.

%If you find this code useful for your research, please cite our paper:
%Antony Lam, Imari Sato and Yoichi Sato. "Denoising Hyperspectral Images
%Using Spectral Domain Statistics." In IEEE International Conference on
%Pattern Recognition (ICPR), IEEE, 2012.
%**************************************************************************

%This function computes SNR as described in:
%G. Chen and S.-E. Qian. "Denoising of Hyperspectral Imagery Using
%Principal Component Analysis and Wavelet Shrinkage." IEEE Transacations on
%Geoscience and Remote Sensing, IEEE, 2011.
function [snr,mask]=SNRHyperspecCube(noiseless,noisy,mask)
if nargin>2
    %Erase irrelevant portions of the hyperspectral cube.
    %mask is a binary 2D matrix indicating which spatial locations are
    %relevant with the value "true."
    mask = reshape((repmat(~mask(:),1,size(noiseless,1)))',size(noiseless));
    noiseless(mask(:)) = [];
    noisy(mask(:)) = [];
end
snr = sum(noiseless(:).^2)/sum((noiseless(:) - noisy(:)).^2);