%**************************************************************************
%Written by Antony Lam (antony@nii.ac.jp)
%National Institute of Informatics, Tokyo, Japan
%September 28, 2013

%Copyright (c) 2013 National Institute of Informatics

%This software is released under the terms of the GNU General Public
%License version 3 as published by the Free Software Foundation.
%A copy of the license is included with this software in the file gpl.txt
%and it may also be obtained at http://www.gnu.org/licenses/.

%The software is provided as is without any warranty or guaranteed support
%whatsoever. The National Institute of Informatics cannot be held
%responsible for its use, misuse, or functionality.

%If you find this code useful for your research, please cite our paper:
%Antony Lam, Imari Sato, and Yoichi Sato. "Denoising Hyperspectral Images
%Using Spectral Domain Statistics." In IEEE International Conference on
%Pattern Recognition (ICPR), IEEE, 2012.
%**************************************************************************

%Takes as input a hyperspectral image. Dimension 1 is for
%wavelengths and the other two dimensions are for the rows and columns of
%each image. basis is a set of orthonormal vectors in column format.
function [denoisedImgs,basis]=denoise(noisyImgs,basis,ave)
if nargin<2
    %Compute top k eigenvectors for spectral domain.
    k = 4;
    [basis,~,ave] = pc_evectors(imgs2ColMatrix(noisyImgs,false),k);
end

%Convert to column vectors and compute coefficients.
vecs = reshape(noisyImgs(:),size(noisyImgs,1),size(noisyImgs,2)*size(noisyImgs,3));
ave = repmat(ave,1,size(vecs,2));
vecs = vecs - ave;
coeffs = basis'*vecs;

%Convert column vectors into image stack (coefficient images).
%Then spatially denoise the coefficient images.
coeffs = reshape(coeffs(:),[size(basis,2) size(noisyImgs,2) size(noisyImgs,3)]);
coeffs = bfilterStack(coeffs);
    
%Convert coefficient images back to vectors.
coeffs = reshape(coeffs(:),size(basis,2),size(noisyImgs,2)*size(noisyImgs,3));

%Project onto the orthonormal basis and reconstruct hyperspectral image.
vecs = basis*coeffs + ave;
denoisedImgs = reshape(vecs(:),size(noisyImgs));