function [E_Img]    =    LRTR_Deblur_Main(O_Img, Y_Img, par)

%% Initialization
Average   =   mean(Y_Img,3);                                                       % Calculate the average band for fast spatial non-local searching
[Neighbor_arr, Num_arr, Self_arr] =	NeighborIndex(Average, par);                   % PreCompute the all the patch index in the searching window 

E_Img     =   Y_Img;
S         =   zeros( size(Y_Img) );
J1        =   zeros( size(Y_Img) );

lambda    =   par.lambda;
alpha     =   par.alpha;
Fh        =   par.fft_B; 
Fh_conj   =   par.fft_BT;

for iter = 1 : par.Iter
    
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    %%  Updata the auxilary variables  S
    Numer           =     Fh_conj.* fftn(Y_Img) + alpha * fftn(E_Img + J1/alpha);
    Demon           =     abs(Fh).^2 + alpha ; 
    S               =     real(ifftn(Numer./Demon));   
    S(S<0)          =     min(min(S(S>0)));
    S(S>1)          =     max(max(S(S<1)));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    %%  Estimation of the Low-rank tensor 
    if (mod(iter-1,10)==0)
    Average         =     mean(E_Img,3);                                            % for fast NL patch searching in 2D;
    [CurPat, Mat]	=	  SR_Im2Patch( E_Img, Average, par );                       % image to patch
    if (mod(iter-1,10)==0)
        NL_mat      =     Block_matching(Mat, par, Neighbor_arr, Num_arr, Self_arr);% caculate NL similar patches for each
    end 
    [EPat, EWei]    =     SR_TPatEstimation( NL_mat, Self_arr, CurPat, par);         % estimate the spatial patches
    [S_Img, S_Wei]  =     SR_Patch2Im( EPat, EWei, par.patsize, size(Y_Img,1), size(Y_Img,2), size(Y_Img,3) );
    end
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    %% Estimation of the high-resolution hyperspectral image
    E_Img           =    (lambda * S_Img + alpha * S -  J1)./(lambda * S_Wei + alpha );
   
    %% Updata Lagrangian multipliers J1
    J1              =    J1 + alpha *( E_Img - S );
    alpha           =    1.05 * alpha;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    %%  Calculate the energy and assessments    
    if (mod(iter-1,10)==0)
%         figure,imshow([S(:,:,20),E_Img(:,:,20)],[]);
%         PSNR  = csnr( 256*O_Img,256* S, 0, 0 );
        PSNR  = csnr( 256*O_Img,256* E_Img, 0, 0 );
        SSIM  = cal_ssim( 256*O_Img,256* E_Img, 0, 0 );
        fprintf( 'Iter = %2.3f, PSNR = %2.2f, SSIM = %2.3f \n', iter, PSNR, SSIM);
    end
end


