function [Z, fun]    =    LRTR_SR_Main(LSpa_Img, LSpe_Img, par, Z_ori, sf)

%% Initialization
Average   =   squeeze(mean(reshape(LSpe_Img,[size(LSpe_Img,1), par.h, par.w]),1)); % Calculate the average band for fast spatial non-local searching
[Neighbor_arr, Num_arr, Self_arr] =	NeighborIndex(Average, par);                   % PreCompute the all the patch index in the searching window 

W         =   zeros( size(LSpa_Img,1), par.h*par.w );
S         =   zeros( size(LSpa_Img,1), par.h*par.w );
Z         =   permute(reshape(imresize( permute(reshape(LSpa_Img,[size(LSpa_Img,1), par.h/sf, par.w/sf]),[2,3,1]), sf, 'bicubic'),[ par.h*par.w ,size(LSpa_Img,1)]),[2,1]);
S_Img     =   zeros( par.h, par.w, size(LSpa_Img,1));
S_Wei     =   zeros( par.h, par.w, size(LSpa_Img,1));


J1        =   zeros( size(Z) );
J2        =   zeros( size(Z) );
Ik        =   eye(size(LSpa_Img,1));

lambda    =   par.lambda;
alpha     =   par.alpha;
belta     =   par.belta; 

XHT       =   par.HT(LSpa_Img);
P         =   par.P;
P2        =   P'* P;
PTY       =   P'* LSpe_Img;
fun       =   zeros(par.Iter, 1);

for iter = 1 : par.Iter
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    %%  Updata the auxilary variables W and S

    B     =    (XHT + belta *(Z + J2/belta))'; 
    for  j  =  1 : size( W, 1 )
        [w,~]     =    pcg( @(x)A_x(x, alpha, belta, par.fft_B, par.fft_BT, sf, [par.h, par.w]), B(:,j), 1E-3, 350, [], [], W(j, :)' );
        W(j, :)   =    w';
    end
    S             =    ( P2 +  alpha * Ik )\( PTY + alpha * Z + J1 );
%     S             =    inv( P2 +  alpha * Ik )*( PTY + alpha * Z + J1 );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    %%  Estimation of the Low-rank tensor     
    if (mod(iter-1,10)==0)
    Z1              =     reshape(Z,[size(P,2), par.h, par.w]);                     % transform the unfold matrix into 3D tensor
    Temp            =     permute(Z1, [2,3,1]);                                     % permutate the tensor as the conventional [height, width, bandth]
%     Average         =     squeeze(mean(Z1,1));                                      % for fast NL patch searching in 2D;
    [CurPat, Mat]	=	  SR_Im2Patch( Temp, Average, par );                        % image to patch
    
    if (mod(iter-1,20)==0)
        NL_mat      =     Block_matching(Mat, par, Neighbor_arr, Num_arr, Self_arr);% caculate NL similar patches for each
    end 
    [EPat, EWei]    =     SR_TPatEstimation( NL_mat, Self_arr, CurPat, par);         % estimate the spatial patches
    [S_Img, S_Wei]  =     SR_Patch2Im( EPat, EWei, par.patsize, par.h, par.w, size(P,2) );
    end
    
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    %% Estimation of the high-resolution hyperspectral image
   
    S_Img1          =     permute(reshape(S_Img, [par.h*par.w, size(LSpa_Img,1)]),[2,1]);
    S_Wei1          =     permute(reshape(S_Wei, [par.h*par.w, size(LSpa_Img,1)]),[2,1]);
    Z               =    (lambda * S_Img1 +  alpha * S -  J1 + belta * W -  J2)./(lambda * S_Wei1 + alpha + belta );
    Z(Z<0)          =    0;
    Z(Z>1)          =    1;
    %% Updata Lagrangian multipliers J1 and J2
    J1              =    J1 + alpha *( Z - S );
    J2              =    J2 + belta *( Z - W );
    alpha           =    1.05 * alpha;
    belta           =    1.05 * belta;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    %%  Calculate the energy and assessments       
    if mod(iter-1,10)==0
        MSE         =    mean( mean( (Z_ori-Z).^2 ) );
        PSNR        =    10*log10(1/MSE);                
        disp( sprintf('Iter %d, RMSE = %3.3f, PSNR = %3.2f', iter, sqrt(MSE)*255, PSNR) );                        
    end   
    
end
% Z         =    alternating_back_projection( Z, LSpa_Img, LSpe_Img, par.P, create_H([par.h, par.w], sf), par );
MSE         =    mean( mean( (Z_ori-Z).^2 ) );
PSNR        =    10*log10(1/MSE);
disp( sprintf('Iter %d, RMSE = %3.3f, PSNR = %3.2f', iter, sqrt(MSE)*255, PSNR) );
return;

